@@ -1,5 +1,6 @@
 from __future__ import absolute_import, print_function
 
+import getpass
 import os
 import random
 import socket
@@ -23,7 +24,8 @@ import dtale.global_state as global_state
 from dtale import dtale
 from dtale.cli.clickutils import retrieve_meta_info_and_version, setup_logging
 from dtale.utils import (DuplicateDataError, build_shutdown_url, build_url,
-                         dict_merge, get_host, running_with_flask_debug)
+                         dict_merge, fix_url_path, get_host,
+                         is_app_root_defined, running_with_flask_debug)
 from dtale.views import DtaleData, head_data_id, is_up, kill, startup
 
 if PY3:
@@ -34,6 +36,7 @@ else:
 logger = getLogger(__name__)
 
 USE_NGROK = False
+JUPYTER_SERVER_PROXY = False
 ACTIVE_HOST = None
 ACTIVE_PORT = None
 
@@ -63,15 +66,15 @@ class DtaleFlaskTesting(FlaskClient):
         self.host = kwargs.pop('hostname', 'localhost')
         self.port = kwargs.pop('port', str(random.randint(0, 65535))) or str(random.randint(0, 65535))
         super(DtaleFlaskTesting, self).__init__(*args, **kwargs)
+        self.application.config['SERVER_NAME'] = '{host}:{port}'.format(host=self.host, port=self.port)
+        self.application.config['SESSION_COOKIE_DOMAIN'] = 'localhost.localdomain'
 
     def get(self, *args, **kwargs):
         """
         :param args: Optional arguments to be passed to :meth:`flask:flask.FlaskClient.get`
         :param kwargs: Optional keyword arguments to be passed to :meth:`flask:flask.FlaskClient.get`
         """
-        return super(DtaleFlaskTesting, self).get(
-            base_url='http://{host}:{port}'.format(host=self.host, port=self.port), *args, **kwargs
-        )
+        return super(DtaleFlaskTesting, self).get(url_scheme='http', *args, **kwargs)
 
 
 class DtaleFlask(Flask):
@@ -86,7 +89,7 @@ class DtaleFlask(Flask):
     :param kwargs: Optional keyword arguments to be passed to :class:`flask:flask.Flask`
     """
 
-    def __init__(self, import_name, reaper_on=True, url=None, *args, **kwargs):
+    def __init__(self, import_name, reaper_on=True, url=None, app_root=None, *args, **kwargs):
         """
         Constructor method
         :param reaper_on: whether to run auto-reaper subprocess
@@ -97,8 +100,21 @@ class DtaleFlask(Flask):
         self.base_url = url
         self.shutdown_url = build_shutdown_url(url)
         self.port = None
+        self.app_root = app_root
         super(DtaleFlask, self).__init__(import_name, *args, **kwargs)
 
+    def update_template_context(self, context):
+        super(DtaleFlask, self).update_template_context(context)
+        if self.app_root is not None:
+            context['url_for'] = self.url_for
+
+    def url_for(self, endpoint, *args, **kwargs):
+        if self.app_root is not None and endpoint == 'static':
+            if 'filename' in kwargs:
+                return fix_url_path('{}/{}'.format(self.app_root, kwargs["filename"]))
+            return fix_url_path('{}/{}'.format(self.app_root, args[0]))
+        return url_for(endpoint, *args, **kwargs)
+
     def run(self, *args, **kwargs):
         """
         :param args: Optional arguments to be passed to :meth:`flask:flask.run`
@@ -110,7 +126,7 @@ class DtaleFlask(Flask):
         self.build_reaper()
         super(DtaleFlask, self).run(use_reloader=kwargs.get('debug', False), *args, **kwargs)
 
-    def test_client(self, reaper_on=False, port=None, *args, **kwargs):
+    def test_client(self, reaper_on=False, port=None, app_root=None, *args, **kwargs):
         """
         Overriding Flask's implementation of test_client so we can specify ports for testing and
         whether auto-reaper should be running
@@ -125,6 +141,10 @@ class DtaleFlask(Flask):
         :rtype: :class:`dtale.app.DtaleFlaskTesting`
         """
         self.reaper_on = reaper_on
+        self.app_root = app_root
+        if app_root is not None:
+            self.config['APPLICATION_ROOT'] = app_root
+            self.jinja_env.globals['url_for'] = self.url_for
         self.test_client_class = DtaleFlaskTesting
         return super(DtaleFlask, self).test_client(*args, **dict_merge(kwargs, dict(port=port)))
 
@@ -172,7 +192,7 @@ class DtaleFlask(Flask):
         return super(DtaleFlask, self).get_send_file_max_age(name)
 
 
-def build_app(url, host=None, reaper_on=True, hide_shutdown=False, github_fork=False):
+def build_app(url, host=None, reaper_on=True, hide_shutdown=False, github_fork=False, app_root=None):
     """
     Builds :class:`flask:flask.Flask` application encapsulating endpoints for D-Tale's front-end
 
@@ -180,13 +200,20 @@ def build_app(url, host=None, reaper_on=True, hide_shutdown=False, github_fork=F
     :rtype: :class:`dtale.app.DtaleFlask`
     """
 
-    app = DtaleFlask('dtale', reaper_on=reaper_on, static_url_path='', url=url, instance_relative_config=False)
+    app = DtaleFlask('dtale', reaper_on=reaper_on, static_url_path='', url=url, instance_relative_config=False,
+                     app_root=app_root)
     app.config['SECRET_KEY'] = 'Dtale'
     app.config['HIDE_SHUTDOWN'] = hide_shutdown
     app.config['GITHUB_FORK'] = github_fork
 
     app.jinja_env.trim_blocks = True
     app.jinja_env.lstrip_blocks = True
+
+    if app_root is not None:
+        app.config['APPLICATION_ROOT'] = app_root
+        app.jinja_env.globals['url_for'] = app.url_for
+    app.jinja_env.globals['is_app_root_defined'] = is_app_root_defined
+
     app.register_blueprint(dtale)
 
     compress = Compress()
@@ -209,7 +236,7 @@ def build_app(url, host=None, reaper_on=True, hide_shutdown=False, github_fork=F
 
         :return: image/png
         """
-        return redirect(url_for('static', filename='images/favicon.ico'))
+        return redirect(app.url_for('static', filename='images/favicon.ico'))
 
     @app.route('/missing-js')
     def missing_js():
@@ -299,7 +326,7 @@ def build_app(url, host=None, reaper_on=True, hide_shutdown=False, github_fork=F
             # Filter out rules we can't navigate to in a browser
             # and rules that require parameters
             if "GET" in rule.methods and has_no_empty_params(rule):
-                url = url_for(rule.endpoint, **(rule.defaults or {}))
+                url = app.url_for(rule.endpoint, **(rule.defaults or {}))
                 links.append((url, rule.endpoint))
         return jsonify(links)
 
@@ -404,9 +431,23 @@ def find_free_port():
     return base
 
 
+def build_startup_url_and_app_root(app_root=None):
+    url = build_url(ACTIVE_PORT, ACTIVE_HOST)
+    final_app_root = app_root
+    if final_app_root is None and JUPYTER_SERVER_PROXY:
+        final_app_root = '/user/{}/proxy/'.format(getpass.getuser())
+    if final_app_root is not None:
+        if JUPYTER_SERVER_PROXY:
+            final_app_root = fix_url_path('{}/{}'.format(final_app_root, ACTIVE_PORT))
+            return final_app_root, final_app_root
+        else:
+            return fix_url_path('{}/{}'.format(url, final_app_root)), final_app_root
+    return url, final_app_root
+
+
 def show(data=None, host=None, port=None, name=None, debug=False, subprocess=True, data_loader=None,
          reaper_on=True, open_browser=False, notebook=False, force=False, context_vars=None, ignore_duplicate=False,
-         **kwargs):
+         app_root=None, **kwargs):
     """
     Entry point for kicking off D-Tale :class:`flask:flask.Flask` process from python process
 
@@ -452,7 +493,7 @@ def show(data=None, host=None, port=None, name=None, debug=False, subprocess=Tru
 
         ..link displayed in logging can be copied and pasted into any browser
     """
-    global ACTIVE_HOST, ACTIVE_PORT, USE_NGROK
+    global ACTIVE_HOST, ACTIVE_PORT, USE_NGROK, JUPYTER_SERVER_PROXY
 
     try:
         logfile, log_level, verbose = map(kwargs.get, ['logfile', 'log_level', 'verbose'])
@@ -469,10 +510,11 @@ def show(data=None, host=None, port=None, name=None, debug=False, subprocess=Tru
         else:
             initialize_process_props(host, port, force)
 
-        url = build_url(ACTIVE_PORT, ACTIVE_HOST)
-        instance = startup(url, data=data, data_loader=data_loader, name=name, context_vars=context_vars,
+        app_url = build_url(ACTIVE_PORT, ACTIVE_HOST)
+        startup_url, final_app_root = build_startup_url_and_app_root(app_root)
+        instance = startup(startup_url, data=data, data_loader=data_loader, name=name, context_vars=context_vars,
                            ignore_duplicate=ignore_duplicate)
-        is_active = not running_with_flask_debug() and is_up(url)
+        is_active = not running_with_flask_debug() and is_up(app_url)
         if is_active:
             def _start():
                 if open_browser:
@@ -484,7 +526,8 @@ def show(data=None, host=None, port=None, name=None, debug=False, subprocess=Tru
                 thread.start()
 
             def _start():
-                app = build_app(url, reaper_on=reaper_on, host=ACTIVE_HOST)
+                app = build_app(app_url, reaper_on=reaper_on, host=ACTIVE_HOST,
+                                app_root=final_app_root)
                 if debug and not USE_NGROK:
                     app.jinja_env.auto_reload = True
                     app.config['TEMPLATES_AUTO_RELOAD'] = True
@@ -513,7 +556,7 @@ def show(data=None, host=None, port=None, name=None, debug=False, subprocess=Tru
             if notebook:
                 instance.notebook()
         else:
-            logger.info('D-Tale started at: {}'.format(url))
+            logger.info('D-Tale started at: {}'.format(app_url))
             _start()
 
         return instance
@@ -554,7 +597,8 @@ def get_instance(data_id):
     """
     data_id_str = str(data_id)
     if global_state.get_data(data_id_str) is not None:
-        return DtaleData(data_id_str, build_url(ACTIVE_PORT, ACTIVE_HOST))
+        startup_url, _ = build_startup_url_and_app_root()
+        return DtaleData(data_id_str, startup_url)
     return None
 
 
