@@ -1,6 +1,7 @@
 import json
 import math
 import os
+import re
 import traceback
 import urllib
 from logging import getLogger
@@ -25,10 +26,11 @@ from dtale.dash_application.layout import (AGGS, ANIMATE_BY_CHARTS,
                                            ANIMATION_CHARTS, build_error,
                                            test_plotly_version,
                                            update_label_for_freq)
+from dtale.dash_application.topojson_injections import INJECTIONS
 from dtale.utils import (build_code_export, classify_type, dict_merge,
                          divide_chunks, export_to_csv_buffer, find_dtype,
-                         find_dtype_formatter, flatten_lists, get_dtypes,
-                         make_list, run_query)
+                         find_dtype_formatter, fix_url_path, flatten_lists,
+                         get_dtypes, make_list, run_query)
 
 logger = getLogger(__name__)
 
@@ -87,8 +89,11 @@ def chart_url_querystring(params, data=None, group_filter=None):
     if chart_type == 'bar':
         base_props += ['barmode', 'barsort']
     elif chart_type == 'maps':
-        if params.get('map_type') == 'scattergeo':
+        map_type = params.get('map_type')
+        if map_type == 'scattergeo':
             base_props += ['map_type', 'lat', 'lon', 'map_val', 'scope', 'proj']
+        elif map_type == 'mapbox':
+            base_props += ['map_type', 'lat', 'lon', 'map_val', 'mapbox_style']
         else:
             base_props += ['map_type', 'loc_mode', 'loc', 'map_val']
         base_props += ['map_group']
@@ -272,16 +277,20 @@ def chart_wrapper(data_id, data, url_params=None):
 
     def _chart_wrapper(chart, group_filter=None):
         querystring = chart_url_querystring(url_params, data=data, group_filter=group_filter)
+        app_root = url_params.get('app_root') or ''
+
+        def build_url(path, query):
+            return fix_url_path('{}/{}/{}?{}'.format(app_root, path, data_id, query))
         popup_link = html.A(
             [html.I(className='far fa-window-restore mr-4'), html.Span('Popup Chart')],
-            href='/charts/{}?{}'.format(data_id, querystring),
+            href=build_url('/charts', querystring),
             target='_blank',
             className='mr-5'
         )
         copy_link = html.Div(
             [html.A(
                 [html.I(className='ico-link mr-4'), html.Span('Copy Link')],
-                href='/charts/{}?{}'.format(data_id, querystring),
+                href=build_url('/charts', querystring),
                 target='_blank',
                 className='mr-5 copy-link-btn'
             ), html.Div('Copied to clipboard', className="hoverable__content copy-tt-bottom")
@@ -295,12 +304,12 @@ def chart_wrapper(data_id, data, url_params=None):
         )
         export_html_link = html.A(
             [html.I(className='fas fa-file-code mr-4'), html.Span('Export Chart')],
-            href='/dtale/chart-export/{}?{}'.format(data_id, querystring),
+            href=build_url('/dtale/chart-export', querystring),
             className='export-chart-btn mr-5'
         )
         export_csv_link = html.A(
             [html.I(className='fas fa-file-csv mr-4'), html.Span('Export CSV')],
-            href='/dtale/chart-csv-export/{}?{}'.format(data_id, querystring),
+            href=build_url('/dtale/chart-csv-export', querystring),
             className='export-chart-btn'
         )
         links = html.Div(
@@ -944,7 +953,7 @@ def heatmap_builder(data_id, export=False, **inputs):
                 code += agg_code
         if not len(data):
             raise Exception('No data returned for this computation!')
-        check_exceptions(data[dupe_cols], agg not in ['corr', 'raw'], unlimited_data=True)
+        check_exceptions(data[dupe_cols], agg in ['corr', 'raw'], unlimited_data=True)
         dtypes = {c: classify_type(dtype) for c, dtype in get_dtypes(data).items()}
         data_f, _ = chart_formatters(data)
         data = data_f.format_df(data)
@@ -1007,8 +1016,9 @@ def map_builder(data_id, export=False, **inputs):
     try:
         if not valid_chart(**inputs):
             return None, None
-        props = ['map_type', 'loc_mode', 'loc', 'lat', 'lon', 'map_val', 'scope', 'proj', 'agg', 'animate_by']
-        map_type, loc_mode, loc, lat, lon, map_val, scope, proj, agg, animate_by = (inputs.get(p) for p in props)
+        props = ['map_type', 'loc_mode', 'loc', 'lat', 'lon', 'map_val', 'scope', 'proj', 'mapbox_style', 'agg',
+                 'animate_by']
+        map_type, loc_mode, loc, lat, lon, map_val, scope, proj, style, agg, animate_by = (inputs.get(p) for p in props)
         map_group, group_val = (inputs.get(p) for p in ['map_group', 'group_val'])
         raw_data = run_query(
             global_state.get_data(data_id),
@@ -1063,6 +1073,46 @@ def map_builder(data_id, export=False, **inputs):
                 config=dict(topojsonURL='/maps/'),
                 figure=figure_cfg
             )
+        elif map_type == 'mapbox':
+            from dtale.charts.utils import get_mapbox_token
+            data, code = retrieve_chart_data(raw_data, lat, lon, map_val, animate_by, map_group, group_val=group_val)
+            if agg is not None:
+                data, agg_code = build_agg_data(raw_data, lat, lon, {}, agg, z=map_val, animate_by=animate_by)
+                code += agg_code
+
+            mapbox_layout = {'style': style}
+            mapbox_token = get_mapbox_token()
+            if mapbox_token is not None:
+                mapbox_layout['accesstoken'] = mapbox_token
+            # if test_plotly_version('4.5.0') and animate_by is None:
+            #     geo_layout['fitbounds'] = 'locations'
+            if len(mapbox_layout):
+                layout['mapbox'] = mapbox_layout
+
+            chart_kwargs = dict(lon=data[lon], lat=data[lat], mode='markers', marker=dict(color='darkblue'))
+            if map_val is not None:
+                chart_kwargs['text'] = data[map_val]
+                chart_kwargs['marker'] = dict(
+                    color=data[map_val], cmin=data[map_val].min(), cmax=data[map_val].max(),
+                    colorscale=inputs.get('colorscale') or 'Jet',
+                    colorbar_title=map_val
+                )
+            figure_cfg = dict(data=[go.Scattermapbox(**chart_kwargs)], layout=layout)
+            if animate_by is not None:
+                def build_frame(df):
+                    frame = dict(lon=df[lon], lat=df[lat], mode='markers')
+                    if map_val is not None:
+                        frame['text'] = df[map_val]
+                        frame['marker'] = dict(color=df[map_val])
+                    return frame
+
+                update_cfg_w_frames(figure_cfg, *build_map_frames(data, animate_by, build_frame))
+            chart = graph_wrapper(
+                id='scattermapbox-graph',
+                style={'margin-right': 'auto', 'margin-left': 'auto', 'height': '95%'},
+                config=dict(topojsonURL='/maps/'),
+                figure=figure_cfg
+            )
         else:
             data, code = retrieve_chart_data(raw_data, loc, map_val, map_group, animate_by, group_val=group_val)
             if agg is not None:
@@ -1395,13 +1445,8 @@ def export_chart(data_id, params):
 
 
 def map_chart_post_processing(html_str, params):
-    topo_find = (
-        '_.fetchTopojson=function(){var t=this,e=y.getTopojsonPath(t.topojsonURL,t.topojsonName);'
-        'return new Promise(function(r,a){n.json(e,function(n,i){if(n)return 404===n.status?a(new Error(["plotly.js '
-        'could not find topojson file at",e,".","Make sure the *topojsonURL* plot config option","is set properly."]'
-        '.join(" "))):a(new Error(["unexpected error while fetching topojson file at",e].join(" ")));'
-        'PlotlyGeoAssets.topojson[t.topojsonName]=i,r()})})}'
-    )
+    plotly_version = next((v for v in re.findall(r" plotly.js v(\d+.\d+.\d+)", html_str)), None)
+    topo_find = INJECTIONS.get(plotly_version) or INJECTIONS[sorted(INJECTIONS.keys())[-1]]
     if topo_find in html_str:
         map_path = os.path.join(os.path.dirname(__file__), '../static/maps/')
         if params.get('map_type') == 'scattergeo':
@@ -1410,7 +1455,7 @@ def map_chart_post_processing(html_str, params):
             topo_name = 'world_110m'
         else:
             topo_name = 'usa_110m'
-        topo_replace = ['_.fetchTopojson=function(){']
+        topo_replace = ['.fetchTopojson=function(){']
         with open(os.path.join(map_path, '{}.json'.format(topo_name)), 'r') as file:
             data = file.read().replace('\n', '')
             topo_replace.append("PlotlyGeoAssets.topojson['{}'] = {};".format(topo_name, data))
