@@ -9,7 +9,7 @@ from pkg_resources import parse_version
 from dtale.charts.utils import YAXIS_CHARTS, ZAXIS_CHARTS, find_group_vals
 from dtale.utils import (ChartBuildingError, classify_type, dict_merge,
                          flatten_lists, get_dtypes, inner_build_query,
-                         make_list)
+                         is_app_root_defined, make_list)
 
 
 def test_plotly_version(version_num):
@@ -18,7 +18,7 @@ def test_plotly_version(version_num):
     return parse_version(plotly.__version__) >= parse_version(version_num)
 
 
-def base_layout(github_fork, **kwargs):
+def base_layout(github_fork, app_root, **kwargs):
     """
     Base layout to be returned by :meth:`dtale.dash_application.views.DtaleDash.interpolate_index`
 
@@ -29,7 +29,7 @@ def base_layout(github_fork, **kwargs):
     :return: HTML
     :rtype: str
     """
-    back_to_data_padding, github_fork_html = ('', '')
+    back_to_data_padding, github_fork_html, webroot_html = ('', '', '')
     if github_fork:
         back_to_data_padding = 'padding-right: 125px'
         github_fork_html = '''
@@ -37,13 +37,22 @@ def base_layout(github_fork, **kwargs):
                 <a href="https://github.com/man-group/dtale">Fork me on GitHub</a>
             </span>
         '''
+    favicon_path = '../../images/favicon.png'
+    if is_app_root_defined(app_root):
+        webroot_html = '''
+        <script type="text/javascript">
+            window.resourceBaseUrl = '{app_root}';
+        </script>
+        '''.format(app_root=app_root)
+        favicon_path = '{}/images/favicon.png'.format(app_root)
     return '''
         <!DOCTYPE html>
         <html>
             <head>
+                {webroot_html}
                 {metas}
                 <title>D-Tale Charts</title>
-                <link rel="shortcut icon" href="../../images/favicon.png">
+                <link rel="shortcut icon" href="{favicon_path}">
                 {css}
             </head>
             <body>
@@ -72,8 +81,8 @@ def base_layout(github_fork, **kwargs):
                     <script type="text/javascript">
                         const pathSegs = window.location.pathname.split('/');
                         const dataId = pathSegs[pathSegs.length - 1];
-                        const backToData = () => window.open('/dtale/main/' + dataId);
-                        const goToLegacy = () => location.replace('/dtale/popup/charts/' + dataId);
+                        const backToData = () => window.open('{app_root}/dtale/main/' + dataId);
+                        const goToLegacy = () => location.replace('{app_root}/dtale/popup/charts/' + dataId);
                     </script>
                     {renderer}
                     {css}
@@ -88,7 +97,10 @@ def base_layout(github_fork, **kwargs):
         scripts=kwargs['scripts'],
         renderer=kwargs['renderer'],
         back_to_data_padding=back_to_data_padding,
-        github_fork_html=github_fork_html
+        webroot_html=webroot_html,
+        github_fork_html=github_fork_html,
+        app_root=app_root if is_app_root_defined(app_root) else '',
+        favicon_path=favicon_path
     )
 
 
@@ -180,7 +192,11 @@ AGGS = dict(
 FREQS = ['H', 'H2', 'WD', 'D', 'W', 'M', 'Q', 'Y']
 FREQ_LABELS = dict(H='Hourly', H2='Hour', WD='Weekday', W='Weekly', M='Monthly', Q='Quarterly', Y='Yearly')
 
-MAP_TYPES = [dict(value='choropleth'), dict(value='scattergeo', label='ScatterGeo')]
+MAP_TYPES = [
+    dict(value='choropleth', image=True),
+    dict(value='scattergeo', label='ScatterGeo', image=True),
+    dict(value='mapbox')
+]
 SCOPES = ['world', 'usa', 'europe', 'asia', 'africa', 'north america', 'south america']
 PROJECTIONS = ['equirectangular', 'mercator', 'orthographic', 'natural earth', 'kavrayskiy7', 'miller', 'robinson',
                'eckert4', 'azimuthal equal area', 'azimuthal equidistant', 'conic equal area', 'conic conformal',
@@ -189,7 +205,7 @@ PROJECTIONS = ['equirectangular', 'mercator', 'orthographic', 'natural earth', '
 
 
 def build_img_src(proj, img_type='projections'):
-    return '/images/{}/{}.png'.format(img_type, '_'.join(proj.split(' ')))
+    return '../images/{}/{}.png'.format(img_type, '_'.join(proj.split(' ')))
 
 
 def build_proj_hover_children(proj):
@@ -220,6 +236,40 @@ def build_proj_hover(proj):
     )
 
 
+def build_mapbox_token_children():
+    from dtale.charts.utils import get_mapbox_token
+    msg = 'To access additional styles enter a token here...'
+    if get_mapbox_token() is None:
+        msg = 'Change your token here...'
+    return [
+        html.I(className='ico-help-outline', style=dict(color='white')),
+        html.Div(
+            [
+                html.Span('Mapbox Access Token:'),
+                dcc.Input(id='mapbox-token-input', type='text', placeholder=msg, className='form-control',
+                          value='', style={'lineHeight': 'inherit'})
+            ],
+            className='hoverable__content',
+            style=dict(width='20em', right='-1.45em')
+        )
+    ]
+
+
+def build_mapbox_token_hover():
+    return html.Span(
+        [
+            'Style',
+            html.Div(
+                build_mapbox_token_children(),
+                className='ml-3 hoverable',
+                style=dict(borderBottom='none'),
+                id='token-hover'
+            )
+        ],
+        className='input-group-addon'
+    )
+
+
 LOC_MODE_INFO = {
     'ISO-3': dict(
         url=html.A('ISO-3', href='https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3', target='_blank'),
@@ -403,11 +453,25 @@ def build_map_options(df, type='choropleth', loc=None, lat=None, lon=None, map_v
     return loc_options, lat_options, lon_options, val_options
 
 
+def build_mapbox_style_options():
+    from dtale.charts.utils import get_mapbox_token
+    free_styles = ['open-street-map', 'carto-positron', 'carto-darkmatter', 'stamen-terrain', 'stamen-toner',
+                   'stamen-watercolor']
+    token_styles = ['basic', 'streets', 'outdoors', 'light', 'dark', 'satellite', 'satellite-streets']
+    styles = free_styles
+    if get_mapbox_token() is not None:
+        styles += token_styles
+    return [build_option(v) for v in styles]
+
+
 def bar_input_style(**inputs):
     """
     Sets display CSS property for bar chart inputs
     """
-    return dict(display='block' if inputs.get('chart_type') == 'bar' else 'none')
+    chart_type, group_col = (inputs.get(p) for p in ['chart_type', 'group'])
+    show_bar = chart_type == 'bar'
+    show_barsort = show_bar and group_col is None
+    return dict(display='block' if show_bar else 'none'), dict(display='block' if show_barsort else 'none')
 
 
 def colorscale_input_style(**inputs):
@@ -415,9 +479,9 @@ def colorscale_input_style(**inputs):
 
 
 def animate_styles(df, **inputs):
-    chart_type, cpg = (inputs.get(p) for p in ['chart_type', 'cpg'])
+    chart_type, agg, cpg = (inputs.get(p) for p in ['chart_type', 'agg', 'cpg'])
     opts = []
-    if cpg:
+    if cpg or agg in ['pctsum', 'pctct']:
         return dict(display='none'), dict(display='none'), opts
     if chart_type in ANIMATION_CHARTS:
         return dict(display='block'), dict(display='none'), opts
@@ -466,10 +530,11 @@ def build_group_val_options(df, group_cols):
 def build_map_type_tabs(map_type):
     def _build_hoverable():
         for t in MAP_TYPES:
-            yield html.Div([
-                html.Span(t.get('label', t['value'].capitalize())),
-                html.Img(src=build_img_src(t['value'], img_type='map_type'))
-            ], className='col-md-6')
+            if t.get('image', False):
+                yield html.Div([
+                    html.Span(t.get('label', t['value'].capitalize())),
+                    html.Img(src=build_img_src(t['value'], img_type='map_type'))
+                ], className='col-md-6')
 
     return html.Div(
         [dcc.Tabs(
@@ -481,7 +546,7 @@ def build_map_type_tabs(map_type):
             html.Div(list(_build_hoverable()), className='row'),
             className='hoverable__content map-types'
         )],
-        style=dict(paddingLeft=15, borderBottom='none'), className="pr-5 hoverable"
+        style=dict(paddingLeft=15, borderBottom='none', width='20em'), className="hoverable"
     )
 
 
@@ -512,7 +577,7 @@ def charts_layout(df, settings, **inputs):
     show_input = show_input_handler(chart_type)
     show_cpg = show_chart_per_group(**inputs)
     show_yaxis = show_yaxis_ranges(**inputs)
-    bar_style = bar_input_style(**inputs)
+    bar_style, barsort_input_style = bar_input_style(**inputs)
     animate_style, animate_by_style, animate_opts = animate_styles(df, **inputs)
 
     options = build_input_options(df, **inputs)
@@ -532,11 +597,11 @@ def charts_layout(df, settings, **inputs):
     show_map = chart_type == 'maps'
     map_props = ['map_type', 'loc_mode', 'loc', 'lat', 'lon', 'map_val']
     map_type, loc_mode, loc, lat, lon, map_val = (inputs.get(p) for p in map_props)
-    map_scope, proj = (inputs.get(p) for p in ['scope', 'proj'])
+    map_scope, proj, mapbox_style = (inputs.get(p) for p in ['scope', 'proj', 'mapbox_style'])
     loc_options, lat_options, lon_options, map_val_options = build_map_options(df, type=map_type, loc=loc, lat=lat,
                                                                                lon=lon, map_val=map_val)
     cscale_style = colorscale_input_style(**inputs)
-    default_cscale = 'Greens' if chart_type == 'heatmap' else 'Reds'
+    default_cscale = 'Jet' if chart_type == 'heatmap' else 'Reds'
 
     group_val_style, main_input_class = main_inputs_and_group_val_display(inputs)
     group_val = [json.dumps(gv) for gv in inputs.get('group_val') or []]
@@ -686,7 +751,7 @@ def charts_layout(df, settings, **inputs):
                             ),
                             id='map-lat-input',
                             label_class='input-group-addon d-block pt-1 pb-0',
-                            style=show_map_style(map_type == 'scattergeo')
+                            style=show_map_style(map_type in ['scattergeo', 'mapbox'])
                         ),
                         build_input(
                             [html.Div('Lon'), html.Small('(Agg By)')],
@@ -699,7 +764,7 @@ def charts_layout(df, settings, **inputs):
                             ),
                             id='map-lon-input',
                             label_class='input-group-addon d-block pt-1 pb-0',
-                            style=show_map_style(map_type == 'scattergeo')
+                            style=show_map_style(map_type in ['scattergeo', 'mapbox'])
                         ),
                         build_input('Scope', dcc.Dropdown(
                             id='map-scope-dropdown',
@@ -707,6 +772,24 @@ def charts_layout(df, settings, **inputs):
                             style=dict(width='inherit'),
                             value=map_scope or 'world'
                         ), id='map-scope-input', style=show_map_style(map_type == 'scattergeo')),
+                        html.Div(
+                            [
+                                html.Div(
+                                    [
+                                        build_mapbox_token_hover(),
+                                        dcc.Dropdown(
+                                            id='map-mapbox-style-dropdown',
+                                            options=build_mapbox_style_options(),
+                                            style=dict(width='inherit'),
+                                            value=mapbox_style or 'open-street-map'
+                                        )
+                                    ],
+                                    className='input-group mr-3',
+                                )
+                            ],
+                            id='map-mapbox-style-input', className='col-auto',
+                            style=show_map_style(map_type == 'mapbox')
+                        ),
                         html.Div(
                             [
                                 html.Div(
@@ -803,7 +886,7 @@ def charts_layout(df, settings, **inputs):
                         ), className='col-auto addon-min-width', style=bar_style, id='barmode-input'),
                         build_input('Barsort', dcc.Dropdown(
                             id='barsort-dropdown', options=barsort_options, value=inputs.get('barsort')
-                        ), className='col-auto addon-min-width', style=bar_style, id='barsort-input'),
+                        ), className='col-auto addon-min-width', style=barsort_input_style, id='barsort-input'),
                         html.Div(
                             html.Div(
                                 [
