@@ -16,6 +16,7 @@ from dtale.dash_application.layout import (animate_styles, bar_input_style,
                                            build_input_options,
                                            build_loc_mode_hover_children,
                                            build_map_options,
+                                           build_mapbox_style_options,
                                            build_proj_hover_children,
                                            charts_layout,
                                            colorscale_input_style,
@@ -24,7 +25,7 @@ from dtale.dash_application.layout import (animate_styles, bar_input_style,
                                            show_chart_per_group,
                                            show_input_handler,
                                            show_yaxis_ranges)
-from dtale.utils import dict_merge, make_list, run_query
+from dtale.utils import dict_merge, is_app_root_defined, make_list, run_query
 
 logger = getLogger(__name__)
 
@@ -44,10 +45,21 @@ class DtaleDash(dash.Dash):
             '/dash/components_bundle.js', '/dash/custom_bundle.js', '/dist/base_styles_bundle.js'
         ]
 
+        app_root = server.config.get('APPLICATION_ROOT')
+        if is_app_root_defined(app_root):
+
+            def _prepend_app_root(v):
+                return '{}{}'.format(app_root, v)
+            kwargs['requests_pathname_prefix'] = _prepend_app_root(kwargs['routes_pathname_prefix'])
+            kwargs['external_stylesheets'] = [_prepend_app_root(v) for v in kwargs['external_stylesheets']]
+            kwargs['external_scripts'] = [_prepend_app_root(v) for v in kwargs['external_scripts']]
+            kwargs['assets_url_path'] = _prepend_app_root('')
+            kwargs['assets_external_path'] = _prepend_app_root('/assets')
+
         super(DtaleDash, self).__init__(*args, **kwargs)
 
     def interpolate_index(self, **kwargs):
-        return base_layout(self.server.config['GITHUB_FORK'], **kwargs)
+        return base_layout(self.server.config['GITHUB_FORK'], self.server.config.get('APPLICATION_ROOT'), **kwargs)
 
 
 def add_dash(server):
@@ -186,6 +198,7 @@ def init_callbacks(dash_app):
             Output('map-lat-input', 'style'),
             Output('map-lon-input', 'style'),
             Output('map-scope-input', 'style'),
+            Output('map-mapbox-style-input', 'style'),
             Output('map-proj-input', 'style'),
             Output('proj-hover', 'style'),
             Output('proj-hover', 'children'),
@@ -200,16 +213,19 @@ def init_callbacks(dash_app):
             Input('map-lon-dropdown', 'value'),
             Input('map-val-dropdown', 'value'),
             Input('map-scope-dropdown', 'value'),
+            Input('map-mapbox-style-dropdown', 'value'),
             Input('map-proj-dropdown', 'value'),
             Input('map-group-dropdown', 'value')
         ],
         [State('url', 'pathname')]
     )
-    def map_data(map_type, loc_mode, loc, lat, lon, map_val, scope, proj, group, pathname):
+    def map_data(map_type, loc_mode, loc, lat, lon, map_val, scope, style, proj, group, pathname):
         data_id = get_data_id(pathname)
         map_type = map_type or 'choropleth'
         if map_type == 'choropleth':
             map_data = dict(map_type=map_type, loc_mode=loc_mode, loc=loc, map_val=map_val)
+        elif map_type == 'mapbox':
+            map_data = dict(map_type=map_type, lat=lat, lon=lon, map_val=map_val, mapbox_style=style)
         else:
             map_data = dict(map_type=map_type, lat=lat, lon=lon, map_val=map_val, scope=scope, proj=proj)
 
@@ -219,17 +235,27 @@ def init_callbacks(dash_app):
         loc_options, lat_options, lon_options, map_val_options = build_map_options(df, type=map_type, loc=loc,
                                                                                    lat=lat, lon=lon, map_val=map_val)
         choro_style = {} if map_type == 'choropleth' else {'display': 'none'}
+        coord_style = {} if map_type in ['scattergeo', 'mapbox'] else {'display': 'none'}
         scatt_style = {} if map_type == 'scattergeo' else {'display': 'none'}
+        mapbox_style = {} if map_type == 'mapbox' else {'display': 'none'}
         proj_hover_style = {'display': 'none'} if proj is None else dict(borderBottom='none')
         proj_hopver_children = build_proj_hover_children(proj)
         loc_mode_hover_style = {'display': 'none'} if loc_mode is None else dict(borderBottom='none')
         loc_mode_children = build_loc_mode_hover_children(loc_mode)
         return (
-            map_data, loc_options, lat_options, lon_options, map_val_options, choro_style, choro_style, scatt_style,
-            scatt_style, scatt_style, scatt_style, proj_hover_style, proj_hopver_children, loc_mode_hover_style,
-            loc_mode_children
+            map_data, loc_options, lat_options, lon_options, map_val_options, choro_style, choro_style, coord_style,
+            coord_style, scatt_style, mapbox_style, scatt_style, proj_hover_style, proj_hopver_children,
+            loc_mode_hover_style, loc_mode_children
         )
 
+    @dash_app.callback(Output('map-mapbox-style-dropdown', 'options'), [Input('mapbox-token-input', 'value')])
+    def update_mapbox_token(token):
+        from dtale.charts.utils import set_mapbox_token
+
+        if token:
+            set_mapbox_token(token)
+        return build_mapbox_style_options()
+
     @dash_app.callback(
         [
             Output('y-multi-input', 'style'),
@@ -262,7 +288,7 @@ def init_callbacks(dash_app):
         group_style = {'display': 'block' if show_input('group') else 'none'}
         rolling_style = {'display': 'inherit' if agg == 'rolling' else 'none'}
         cpg_style = {'display': 'block' if show_chart_per_group(**inputs) else 'none'}
-        bar_style = bar_input_style(**inputs)
+        bar_style, barsort_style = bar_input_style(**inputs)
         yaxis_style = {'display': 'block' if show_yaxis_ranges(**inputs) else 'none'}
 
         data_id = get_data_id(pathname)
@@ -270,7 +296,7 @@ def init_callbacks(dash_app):
         animate_style, animate_by_style, animate_opts = animate_styles(df, **inputs)
 
         return (
-            y_multi_style, y_single_style, z_style, group_style, rolling_style, cpg_style, bar_style, bar_style,
+            y_multi_style, y_single_style, z_style, group_style, rolling_style, cpg_style, bar_style, barsort_style,
             yaxis_style, animate_style, animate_by_style, animate_opts
         )
 
@@ -331,6 +357,8 @@ def init_callbacks(dash_app):
         all_inputs = dict_merge(inputs, chart_inputs, dict(yaxis=yaxis_data or {}), map_data)
         if all_inputs == last_chart_inputs:
             raise PreventUpdate
+        if is_app_root_defined(dash_app.server.config.get('APPLICATION_ROOT')):
+            all_inputs['app_root'] = dash_app.server.config['APPLICATION_ROOT']
         charts, range_data, code = build_chart(get_data_id(pathname), **all_inputs)
         return charts, all_inputs, range_data, code, get_yaxis_type_tabs(make_list(inputs.get('y') or []))
 
