@@ -7,8 +7,8 @@ import webbrowser
 from builtins import map, range, str, zip
 from logging import getLogger
 
-from flask import (json, make_response, redirect, render_template, request,
-                   url_for)
+from flask import (current_app, json, make_response, redirect, render_template,
+                   request)
 
 import numpy as np
 import pandas as pd
@@ -21,6 +21,7 @@ from dtale.charts.utils import build_base_chart
 from dtale.cli.clickutils import retrieve_meta_info_and_version
 from dtale.column_builders import ColumnBuilder
 from dtale.column_filters import ColumnFilter
+from dtale.column_replacements import ColumnReplacement
 from dtale.dash_application.charts import (build_raw_chart, chart_url_params,
                                            chart_url_querystring, export_chart,
                                            export_chart_data, url_encode_func)
@@ -242,8 +243,10 @@ class DtaleData(object):
             logger.info('in order to use this function, please install IPython')
             return self.data.__repr__()
 
-        while not self.is_up():
+        retries = 0
+        while not self.is_up() and retries < 10:
             time.sleep(0.01)
+            retries += 1
 
         self._notebook_handle = display(
             self._build_iframe(route=route, params=params, width=width, height=height), display_id=True
@@ -409,7 +412,7 @@ def dtype_formatter(data, dtypes, data_ranges, prev_dtypes=None):
             visible = prev_dtypes[col].get('visible', True)
         s = data[col]
         dtype_data = dict(name=col, dtype=dtype, index=col_index, visible=visible,
-                          hasMissing=bool(s.isnull().any()), hasOutliers=False)
+                          hasMissing=int(s.isnull().sum()), hasOutliers=0)
         classification = classify_type(dtype)
         if classification in ['F', 'I'] and not data[col].isnull().all() and col in data_ranges:  # floats/ints
             col_ranges = data_ranges[col]
@@ -419,11 +422,11 @@ def dtype_formatter(data, dtypes, data_ranges, prev_dtypes=None):
             # load outlier information
             o_s, o_e = calc_outlier_range(s)
             if not any((np.isnan(v) or np.isinf(v) for v in [o_s, o_e])):
-                dtype_data['hasOutliers'] = bool(((s < o_s) | (s > o_e)).any())
+                dtype_data['hasOutliers'] += int(((s < o_s) | (s > o_e)).sum())
                 dtype_data['outlierRange'] = dict(lower=o_s, upper=o_e)
 
         if classification == 'S' and not dtype_data['hasMissing']:
-            dtype_data['hasMissing'] = bool((s.str.strip() == '').any())
+            dtype_data['hasMissing'] += int((s.str.strip() == '').sum())
         return dtype_data
     return _formatter
 
@@ -464,7 +467,13 @@ def format_data(data):
         data = data.to_frame(index=False)
 
     index = [str(i) for i in make_list(data.index.name or data.index.names) if i is not None]
-    data = data.reset_index().drop('index', axis=1, errors='ignore')
+    drop = True
+    if not len(index) and not data.index.equals(pd.RangeIndex(0, len(data))):
+        drop = False
+        index = ['index']
+    data = data.reset_index()
+    if drop:
+        data = data.drop('index', axis=1, errors='ignore')
     data.columns = [str(c) for c in data.columns]
     return data, index
 
@@ -562,7 +571,7 @@ def base_render_template(template, data_id, **kwargs):
      - processes
     """
     if not len(os.listdir('{}/static/dist'.format(os.path.dirname(__file__)))):
-        return redirect(url_for('missing_js'))
+        return redirect(current_app.url_for('missing_js'))
     curr_settings = global_state.get_settings(data_id) or {}
     _, version = retrieve_meta_info_and_version('dtale')
     return render_template(
@@ -726,6 +735,41 @@ def update_settings(data_id):
         return jsonify_error(e)
 
 
+@dtale.route('/update-formats/<data_id>')
+def update_formats(data_id):
+    """
+    :class:`flask:flask.Flask` route which updates the "formats" property for global SETTINGS associated w/ the
+    current port
+
+    :param data_id: integer string identifier for a D-Tale process's data
+    :type data_id: str
+    :param all: boolean flag which, if true, tells us we should apply this formatting to all columns with the same
+                data type as our selected column
+    :param col: selected column
+    :param format: JSON string for the formatting configuration we want applied to either the selected column of all
+                   columns with the selected column's data type
+    :return: JSON
+    """
+    try:
+        update_all_dtype = get_bool_arg(request, 'all')
+        col = get_str_arg(request, 'col')
+        col_format = get_json_arg(request, 'format')
+        curr_settings = global_state.get_settings(data_id) or {}
+        updated_formats = {col: col_format}
+        if update_all_dtype:
+            dtypes = global_state.get_dtypes(data_id)
+            col_dtype = next((c['dtype'] for c in dtypes if c['name'] == col), None)
+            updated_formats = {
+                c['name']: col_format for c in global_state.get_dtypes(data_id) if c['dtype'] == col_dtype
+            }
+        updated_formats = dict_merge(curr_settings.get('formats') or {}, updated_formats)
+        updated_settings = dict_merge(curr_settings, dict(formats=updated_formats))
+        global_state.set_settings(data_id, updated_settings)
+        return jsonify(dict(success=True))
+    except BaseException as e:
+        return jsonify_error(e)
+
+
 def refresh_col_indexes(data_id):
     """
     Helper function to sync column indexes to current state of dataframe for data_id.
@@ -908,6 +952,72 @@ def reshape_data(data_id):
         return jsonify_error(e)
 
 
+@dtale.route('/build-replacement/<data_id>')
+def build_replacement(data_id):
+    """
+    :class:`flask:flask.Flask` route to handle the replacement of specific values within a column in a dataframe. Some
+    of the operations the are available are:
+     - spaces: replace values consisting of only spaces with a specific value
+     - value: replace specific values with a specific value or aggregation
+     - strings: replace values which contain a specific character or string (case-insensitive or not) with a
+                       specific value
+     - imputer: replace nan values using sklearn imputers iterative, knn or simple
+
+    :param data_id: integer string identifier for a D-Tale process's data
+    :type data_id: str
+    :param col: string from flask.request.args['col'] of the column to perform replacements upon
+    :param type: string from flask.request.args['type'] of the type of replacement to perform
+                 (spaces/fillna/strings/imputer)
+    :param cfg: dict from flask.request.args['cfg'] of how to calculate the replacements
+    :return: JSON {success: True/False}
+    """
+
+    def build_data_ranges(data, col, dtype):
+        data_ranges = {}
+        if classify_type(dtype) == 'F' and not data[col].isnull().all():
+            try:
+                data_ranges[col] = data[[col]].agg(['min', 'max']).to_dict()[col]
+            except ValueError:
+                pass
+        return data_ranges
+
+    try:
+        data = global_state.get_data(data_id)
+        name = get_str_arg(request, 'name')
+        if name is not None:
+            name = str(name)
+            if name in data.columns:
+                raise Exception("A column named '{}' already exists!".format(name))
+        col = get_str_arg(request, 'col')
+        replacement_type = get_str_arg(request, 'type')
+        cfg = json.loads(get_str_arg(request, 'cfg'))
+
+        builder = ColumnReplacement(data_id, col, replacement_type, cfg)
+        output = builder.build_replacements()
+        dtype = find_dtype(output)
+        curr_dtypes = global_state.get_dtypes(data_id)
+
+        if name is not None:
+            data.loc[:, name] = output
+            dtype_f = dtype_formatter(data, {name: dtype}, build_data_ranges(data, name, dtype))
+            curr_dtypes.append(dtype_f(len(curr_dtypes), name))
+        else:
+            data.loc[:, col] = output
+            dtype_f = dtype_formatter(data, {col: dtype}, build_data_ranges(data, col, dtype))
+            col_index = next((i for i, d in enumerate(curr_dtypes) if d['name'] == col), None)
+            curr_col_dtype = dtype_f(col_index, col)
+            curr_dtypes = [curr_col_dtype if d['name'] == col else d for d in curr_dtypes]
+
+        global_state.set_data(data_id, data)
+        global_state.set_dtypes(data_id, curr_dtypes)
+        curr_history = global_state.get_history(data_id) or []
+        curr_history += [builder.build_code()]
+        global_state.set_history(data_id, curr_history)
+        return jsonify(success=True)
+    except BaseException as e:
+        return jsonify_error(e)
+
+
 @dtale.route('/test-filter/<data_id>')
 def test_filter(data_id):
     """
@@ -1069,10 +1179,17 @@ def outliers(data_id, column):
         iqr_lower, iqr_upper = calc_outlier_range(s)
         formatter = find_dtype_formatter(find_dtype(df[column]))
         outliers = s[(s < iqr_lower) | (s > iqr_upper)].unique()
+        if not len(outliers):
+            return jsonify(outliers=[])
         top = len(outliers) > 100
         outliers = [formatter(v) for v in outliers[:100]]
-        query = '(({column} < {lower}) or ({column} > {upper}))'.format(column=column, lower=json_float(iqr_lower),
-                                                                        upper=json_float(iqr_upper))
+        queries = []
+        if iqr_lower > s.min():
+            queries.append('{column} < {lower}'.format(column=column, lower=json_float(iqr_lower)))
+        if iqr_upper < s.max():
+            queries.append('{column} > {upper}'.format(column=column, upper=json_float(iqr_upper)))
+        query = '(({}))'.format(') or ('.join(queries)) if len(queries) > 1 else queries[0]
+
         code = (
             "s = df['{column}']\n"
             "q1 = s.quantile(0.25)\n"
@@ -1114,7 +1231,7 @@ def rename_col(data_id, column):
         rename = get_str_arg(request, 'rename')
         data = global_state.get_data(data_id)
         if column != rename and rename in data.columns:
-            return jsonify(dict(error='Column name "{}" already exists!'))
+            return jsonify(error='Column name "{}" already exists!')
 
         data = data.rename(columns={column: rename})
         curr_history = global_state.get_history(data_id) or []
@@ -1132,6 +1249,71 @@ def rename_col(data_id, column):
         return jsonify_error(e)
 
 
+@dtale.route('/edit-cell/<data_id>/<column>')
+def edit_cell(data_id, column):
+    try:
+        row_index = get_int_arg(request, "rowIndex")
+        updated = get_str_arg(request, "updated")
+        updated_str = updated
+        curr_settings = global_state.get_settings(data_id)
+        data = run_query(
+            global_state.get_data(data_id),
+            build_query(data_id, curr_settings.get('query')),
+            global_state.get_context_variables(data_id),
+            ignore_empty=True,
+        )
+        dtype = find_dtype(data[column])
+
+        code = []
+        if updated in ['nan', 'inf']:
+            updated_str = 'np.{}'.format(updated)
+            updated = getattr(np, updated)
+            data.loc[row_index, column] = updated
+            code.append("df.loc[{row_index}, '{column}'] = {updated}".format(row_index=row_index, column=column,
+                                                                             updated=updated_str))
+        else:
+            classification = classify_type(dtype)
+            if classification == 'B':
+                updated = updated.lower() == 'true'
+                updated_str = str(updated)
+            elif classification == 'I':
+                updated = int(updated)
+            elif classification == 'F':
+                updated = float(updated)
+            elif classification == 'D':
+                updated_str = 'pd.Timestamp({})'.format(updated)
+                updated = pd.Timestamp(updated)
+            elif classification == 'TD':
+                updated_str = 'pd.Timedelta({})'.format(updated)
+                updated = pd.Timedelta(updated)
+            else:
+                if dtype.startswith('category') and updated not in data[column].unique():
+                    data[column].cat.add_categories(updated, inplace=True)
+                    code.append("data['{column}'].cat.add_categories('{updated}', inplace=True)".format(
+                        column=column, updated=updated))
+                updated_str = "'{}'".format(updated)
+            data.at[row_index, column] = updated
+            code.append("df.at[{row_index}, '{column}'] = {updated}".format(row_index=row_index, column=column,
+                                                                            updated=updated_str))
+        curr_history = global_state.get_history(data_id) or []
+        curr_history += code
+        global_state.set_history(data_id, curr_history)
+
+        data = global_state.get_data(data_id)
+        dtypes = global_state.get_dtypes(data_id)
+        ranges = {}
+        try:
+            ranges[column] = data[[column]].agg(['min', 'max']).to_dict()[column]
+        except ValueError:
+            pass
+        dtype_f = dtype_formatter(data, {column: dtype}, ranges)
+        dtypes = [dtype_f(dt['index'], column) if dt['name'] == column else dt for dt in dtypes]
+        global_state.set_dtypes(data_id, dtypes)
+        return jsonify(success=True)
+    except BaseException as e:
+        return jsonify_error(e)
+
+
 @dtale.route('/column-filter-data/<data_id>/<column>')
 def get_column_filter_data(data_id, column):
     try:
@@ -1329,7 +1511,7 @@ def get_column_analysis(data_id):
                 code.append((
                     "chart['ordinal'] = ordinal_data\n"
                     "chart = chart.sort_values('ordinal')"
-                ).format(col=selected_col, ordinal=ordinal_col, agg=ordinal_agg))
+                ))
             hist.index.name = 'labels'
             hist = hist.reset_index()
             hist, top = handle_top(hist, get_int_arg(request, 'top'))
@@ -1357,7 +1539,7 @@ def get_column_analysis(data_id):
         elif data_type == 'histogram':
             hist_data, hist_labels = np.histogram(data, bins=bins)
             hist_data = [json_float(h) for h in hist_data]
-            hist_labels = ['{0:.1f}'.format(l) for l in hist_labels]
+            hist_labels = ['{0:.1f}'.format(lbl) for lbl in hist_labels]
             code.append("chart = np.histogram(df[~pd.isnull(df['{col}'])][['{col}']], bins={bins})".format(
                 col=selected_col, bins=bins))
             desc, desc_code = load_describe(data[selected_col])
